#define _WIN32_WINNT 0x0400
#include <Windows.h>

//On release, stdlibs are not included, so a different initialization function is used (Removes 7KB from executable)
#ifdef _DEBUG
#define MainFunc() int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
#else
#define MainFunc() int WinMainCRTStartup()
#endif

//Global variables
boolean ToggleDown; //If the toggle key is currently down
char ToggleNotUsed; //If a remap was used last time the toggle was pressed [0] (if not [2], the next time the toggle button is pressed, the key is processed normally)
char RemapKeys[255]; //Remapped keys, 0 means no remap
const unsigned char ToggleKey=(char)220; // "\"

//Process keyboard strokes
HHOOK hhkLowLevelKybd;
LRESULT CALLBACK LowLevelKeyboardProc(int nCode, WPARAM wParam, LPARAM lParam)
{
	PKBDLLHOOKSTRUCT p=(PKBDLLHOOKSTRUCT)lParam; //Get the keyboard hook structure

	if(nCode!=HC_ACTION || p->scanCode==1) //If a key is not pressed, perform action normally
		return(CallNextHookEx(NULL, nCode, wParam, lParam));

	switch (wParam)
	{
		case WM_KEYDOWN:
		case WM_SYSKEYDOWN:
			if(ToggleDown && RemapKeys[p->vkCode]!=0) //If toggle key is down and a key with a remap is pressed...
			{
				keybd_event(RemapKeys[p->vkCode], 1, 0, 0); //Remap the key
				ToggleNotUsed=0; //Toggle key remap used
				return TRUE;
			}
			else if(p->vkCode==ToggleKey) //Otherwise, if the toggle key was pressed...
			{
				if(ToggleNotUsed<2) //If previous toggle did not had a remap... (otherwise, it processes the key stroke normally)
				{
					if(ToggleNotUsed==0) //First time the key is pressed, not a repeat stroke...
						ToggleDown=ToggleNotUsed=1; //Everything is true...
					return TRUE;
				}
			}
			else
				ToggleNotUsed=0; //Normal key used between toggle key, so double toggle not pressed
			break;
		case WM_KEYUP:
		case WM_SYSKEYUP:
			if(p->vkCode==ToggleKey) //If toggle key is let up...
				if(ToggleDown) //If toggle was being used, do not process keystroke and turn toggle off
				{
					ToggleNotUsed=(ToggleNotUsed==1 ? 2 : 0);
					ToggleDown=FALSE;
					return TRUE;
				}
				else //Otherwise, previous toggle did not use a remap, so this operation processes the key normally
					ToggleNotUsed=0; //We are no longer toggling
	}

	return CallNextHookEx(NULL, nCode, wParam, lParam); //If no action was taken, process key normally
}

MainFunc()
{
	//Initialize data
	unsigned char KeysToRemap[]={0x09,0x0D,0x51,0x50,0x57,0x4F,0x45,0x49,0x52,0x55,0x54,0x59,0x41,0xBA,0x53,0x4C,0x44,0x4B,0x46,0x4A,0x47,0x48,0x5A,0xBF,0x58,0xBE,0x43,0xBC,0x56,0x4D,0x42,0x4E,0x31,0x30,0x32,0x39,0x33,0x38,0x34,0x37,0x35,0x36,0xC0,0x08}; //Remap key pairs
	int i;
	for(i=0;i<255;i++) //Zero out all key remaps
		RemapKeys[i]=0;
	for(i=0;i<sizeof(KeysToRemap);i+=2) //Remap key pairs to each other
	{
		RemapKeys[KeysToRemap[i]]=KeysToRemap[i+1];
		RemapKeys[KeysToRemap[i+1]]=KeysToRemap[i];
	}
	ToggleDown=FALSE;
	ToggleNotUsed=0;

	//Main process commands
	hhkLowLevelKybd=SetWindowsHookEx(WH_KEYBOARD_LL, LowLevelKeyboardProc, GetModuleHandle(NULL), 0); //Hook the keyboard process
	MessageBox(NULL, TEXT("Click \"Ok\" to terminate this application and disable this function."),TEXT("HalfKey v1.0 by Dakusan"), MB_OK); //Wait for OK signal to terminate

	//Exit
	UnhookWindowsHookEx(hhkLowLevelKybd);
	return 0;
}
